"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var ConfigService_1;
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const dotenv_1 = require("dotenv");
const path = require("path");
const assert = require("assert");
const glob_1 = require("glob");
const get = require("lodash.get");
const constants_1 = require("./constants");
const exceptions_1 = require("./exceptions");
const utils_1 = require("./utils");
const config_1 = require("./config");
let ConfigService = ConfigService_1 = class ConfigService {
    /**
     * @param moduleRef
     */
    constructor(moduleRef) {
        this.moduleRef = moduleRef;
    }
    /**
     * @param options
     */
    static loadDotEnv(options) {
        if (options !== false)
            dotenv_1.config(options || this.dotenvOptions);
    }
    /**
     * @param dir
     */
    static root(dir = '') {
        const rootPath = this.rootPath || path.resolve(process.cwd());
        return path.resolve(rootPath, dir);
    }
    /**
     * @param dir
     */
    static resolveRootPath(dir) {
        assert.ok(path.isAbsolute(dir), 'Start path must be an absolute path.');
        if (!this.rootPath) {
            const root = this.root();
            let workingDir = dir;
            let parent = path.dirname(dir);
            while (workingDir !== root && parent !== root && parent !== workingDir) {
                workingDir = parent;
                parent = path.dirname(workingDir);
            }
            this.rootPath = workingDir;
        }
        return this;
    }
    /**
     * @param glob
     */
    static getConfigFiles(glob) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, rejects) => {
                new glob_1.Glob(glob, {
                    cwd: this.root(),
                }, (err, matches) => {
                    if (err) {
                        rejects(err);
                    }
                    else {
                        resolve(matches);
                    }
                });
            });
        });
    }
    /**
     * @param glob
     */
    static createProviders(glob) {
        return __awaiter(this, void 0, void 0, function* () {
            ConfigService_1.mode = 'async';
            const files = yield this.getConfigFiles(glob);
            const providers = [];
            files.forEach(file => {
                providers.push(this.loadFile(file));
            });
            return providers;
        });
    }
    /**
     * @param file
     */
    static loadFile(file) {
        const required = require(file);
        const provider = utils_1.createProvider(required.default, file);
        if (!provider.hasOwnProperty('useClass')) {
            this.setReference(required.default.__name || required.default.__provide || path.basename(file, '.' + file.split('.').pop()), typeof provider.provide === 'function' ? provider.provide['prototype'] : provider.provide);
        }
        return provider;
    }
    /**
     * @param token
     */
    findConfigProvider(token) {
        return this.moduleRef.get(token);
    }
    /**
     * @param name
     */
    resolveTokenFromName(name) {
        const token = this.has(name) ? ConfigService_1.tokenReferences[name] : null;
        if (token === null) {
            throw new exceptions_1.UnkownConfigProvider(name);
        }
        return token;
    }
    /**
     * @param name
     */
    has(name) {
        return Object.keys(ConfigService_1.tokenReferences).includes(name);
    }
    /**
     * @param name
     * @param token
     */
    static setReference(name, token) {
        ConfigService_1.tokenReferences[name] = token;
        return this;
    }
    /**
     * @param pattern
     */
    getNameAndAshPatternFromPattern(pattern) {
        const parts = pattern.split('.');
        return [parts.shift(), parts.join('.')];
    }
    /**
     * @param pattern
     * @param value
     */
    get(pattern, value = undefined) {
        let provider;
        let ashPattern;
        let name;
        if (ConfigService_1.mode === 'sync') {
            [name, ashPattern] = this.getNameAndAshPatternFromPattern(pattern);
            provider = this.findConfigProvider(`${constants_1.TOKEN_PREFIX}${ashPattern ? name : ''}`);
            if (ashPattern === '') {
                ashPattern = name;
            }
        }
        else {
            [name, ashPattern] = this.getNameAndAshPatternFromPattern(pattern);
            provider = this.findConfigProvider(this.resolveTokenFromName(name));
        }
        if (!provider) {
            return value;
        }
        if (provider instanceof config_1.Config && ashPattern) {
            return provider.get(ashPattern, value);
        }
        const result = ashPattern ? get(provider, ashPattern) : provider;
        return result ? result : value;
    }
};
ConfigService.tokenReferences = {};
ConfigService.mode = 'sync';
ConfigService.dotenvOptions = {};
ConfigService = ConfigService_1 = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [core_1.ModuleRef])
], ConfigService);
exports.ConfigService = ConfigService;
